/*

This file is part of jHaushalt.

jHaushalt is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

jHaushalt is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with jHaushalt; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

(C)opyright 2002-2005 Dr. Lars H. Hahn

*/

package haushalt.auswertung;

import haushalt.auswertung.bloecke.AbstractBlock;
import haushalt.daten.Datenbasis;
import haushalt.daten.Datum;
import haushalt.gui.Haushalt;
import haushalt.gui.generischerdialog.AbstractGDPane;
import haushalt.gui.generischerdialog.GenerischerDialog;
import haushalt.gui.generischerdialog.TextGDP;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.Graphics;
import java.awt.Image;
import java.awt.Toolkit;
import java.awt.print.PageFormat;
import java.awt.print.PrinterException;
import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.IOException;
import java.net.URL;
import java.net.URLClassLoader;
import java.util.ArrayList;
import java.util.Date;

import javax.swing.JFrame;
import javax.swing.JTextField;

/**
 * Abstrakte Klasse, die Auswertungen standardisiert.
 * Die Auswertung wird hierzu in einzelne Blcke aufgeteilt, diese
 * knnen fr unterschiedliche Auswertungen wiederverwertet werden.
 * Darber hinaus werden die Parameter der Auswertung ber einen
 * generischen Dialog eingestellt, der ebenfalls wiederverwendet werden
 * kann. In der abgeleiteten Auswertung muss lediglich die Methode
 * 'berechneAuswertung' berladen werden.
 * @see AbstractBlockAuswertung#berechneAuswertung(Object[])
 * 
 * @author Dr. Lars H. Hahn
 * @version 2.0/2004.08.22
 */
public abstract class AbstractBlockAuswertung extends AbstractAuswertung {
  private final boolean DEBUG = false;

  protected final Haushalt haushalt;
  private final ArrayList bloecke = new ArrayList();
  private final Image wasserzeichen;

  public AbstractBlockAuswertung(Haushalt haushalt, Datenbasis db, String name) {
    super(db, name);
    this.haushalt = haushalt;
		URLClassLoader urlLoader = (URLClassLoader)getClass().getClassLoader();
		URL fileLoc = urlLoader.findResource("res/jhh-grau.gif");
		wasserzeichen = Toolkit.getDefaultToolkit().createImage(fileLoc);
  }

  public void addDokumentenBlock(AbstractBlock block) {
    bloecke.add(block);
    if(DEBUG)
      System.out.println("AbstractBlockAuswertung: Block hinzugefgt.");
  }
  
  public void loescheBloecke() {
    bloecke.clear();
    if(DEBUG)
      System.out.println("AbstractBlockAuswertung: Alle Blcke gelscht.");
  }

  public void paintComponent(Graphics g) {
    setBackground(Color.white);
    super.paintComponent(g); // paint background
    int breite = getSize().width - 2;
    int y = 0;
    for(int i=0; i<bloecke.size(); i++) {
      AbstractBlock block = (AbstractBlock) bloecke.get(i);
      y += block.paint(g, 0, y, breite);
    }
    setPreferredSize(new Dimension(breite, y));
    revalidate();
  }
  
  private int seitenZaehler = 0;
  private int blockZaehler = 0;
  private int zeilenZaehler = 0;
  private int blockStart = 0;
  private int zeilenStart = 0;

  public int print(Graphics g, PageFormat seitenFormat, int gesuchteSeite) throws PrinterException {
    if(DEBUG)
      System.out.println("AbstractBlockAuswertung: Drucke Seite "+gesuchteSeite);
    if(gesuchteSeite == 0) {
      // Die erste Seite ist gewnscht => alle Zhler zurcksetzen
      seitenZaehler = 0;
      blockZaehler = 0;
      zeilenZaehler = 0;
      blockStart = 0;
      zeilenStart = 0;
    }
    else if(gesuchteSeite == seitenZaehler) {
      // Die aktuelle Seite wird wiederholt abgefragt => Zhler wieder auf die
      // Anfangswerte
      blockZaehler = blockStart;
      zeilenZaehler = zeilenStart;
    }
    else if(gesuchteSeite == (seitenZaehler+1)) {
      // Die nchste Seite ist gewnscht
      seitenZaehler ++;
      blockStart = blockZaehler;
      zeilenStart = zeilenZaehler;
    }
    else
      throw new PrinterException("Seiten nicht in der richtigen Reihenfolge angefodert.");
    
    if(blockZaehler == bloecke.size())
      return NO_SUCH_PAGE;
    
    int hoehe = (int)seitenFormat.getImageableHeight() - 2;
    int breite = (int)seitenFormat.getImageableWidth() - 2;
    int xStart = (int)seitenFormat.getImageableX() + 1;
    int yStart = (int)seitenFormat.getImageableY() + 1;

    // Fusszeile drucken
    g.setFont(new Font(haushalt.getFontname(), Font.PLAIN, haushalt.getFontgroesse()-2));
    int textHoehe = g.getFontMetrics().getHeight();
    int bildBreite = textHoehe*169 / 32;
    g.drawImage(wasserzeichen, xStart, yStart+hoehe-textHoehe, bildBreite, textHoehe, Color.white, null);
    String seitenzahl = ""+new Datum()+" / Seite "+(gesuchteSeite+1);
    int y = yStart + hoehe - g.getFontMetrics().getDescent();
    int x = xStart + breite - g.getFontMetrics().stringWidth(seitenzahl);
    g.drawString(seitenzahl, x, y);
    hoehe -= textHoehe;
    
    y = 0;
    int verbrauchteHoehe;
    do {
      if(DEBUG)
        System.out.println("AbstractBlockAuswertung: Block "+blockZaehler+" wird gedruckt.");
      AbstractBlock block = (AbstractBlock) bloecke.get(blockZaehler);
      verbrauchteHoehe = block.print(g, zeilenZaehler, xStart, yStart+y, hoehe-y, breite);
      if(verbrauchteHoehe == -1) {
        // keine Zeilen mehr in diesem Block
        blockZaehler ++;
        zeilenZaehler = 0;
      }
      else if((verbrauchteHoehe == 0) && (y == 0))
        throw new PrinterException("Block ist zu hoch fr die Seite.");
      else {
        y += verbrauchteHoehe;
        zeilenZaehler ++;
      }
    } while((blockZaehler < bloecke.size()) && (verbrauchteHoehe != 0));

    return PAGE_EXISTS;
  }

  private AbstractGDPane[] panes;
  private TextGDP namenPane;
  private GenerischerDialog dlgEigenschaften;
  
	abstract protected String berechneAuswertung(Object[] werte);

  /**
   * Ermglicht die Auswertung neu zu berechnen, wenn sich die allgemeinen
   * Eigenschaften ndern.
   */
	public final String berechneAuswertung() {
    Object[] werte = new Object[panes.length];
    long start = new Date().getTime();
    for(int i=0; i<panes.length; i++)
      werte[i] = panes[i].getWert();
    String titel = berechneAuswertung(werte);
    if(DEBUG)
      System.out.println(toString()+" in "+(new Date().getTime()-start)+" ms berechnet.");
    return titel;
  }
	
	public final void erzeugeEigenschaften(JFrame frame, String ueberschrift, AbstractGDPane[] gdPanes) {
		this.panes = gdPanes;
    Object[] werte = new Object[panes.length];
    dlgEigenschaften = new GenerischerDialog(ueberschrift, frame);
		for(int i=0; i<panes.length; i++) {
		  dlgEigenschaften.addPane(panes[i]);   
		  werte[i] = panes[i].getWert();
		}
		namenPane = new TextGDP("Name der Auswertung:", toString());
		dlgEigenschaften.addPane(namenPane);
  }
  
  public final boolean zeigeEigenschaften() {
		for(int i=0; i<panes.length; i++)
		  panes[i].refreshRegisterUndKategorien(db.getKategorien(), db.getRegisterNamen());
    if(dlgEigenschaften.showDialog()) {
      String titel = berechneAuswertung();
      // Wenn der Benutzer die Auswertung nicht benannt hat, wird sie mit
      // einem automatischen Namen versehen.
      if(namenPane.getWert().toString().startsWith("Unbenannt")) {
        ((JTextField)namenPane.getZentraleKomponente()).setText(titel);
        setAuswertungName(titel);
      }
      else
        setAuswertungName(""+namenPane.getWert());
      return true;
    }
    return false;
  }

  public final void laden(DataInputStream in) throws IOException {
		for(int i=0; i<panes.length; i++)
		  panes[i].laden(in);
  }

  public final void speichern(DataOutputStream out) throws IOException {
		for(int i=0; i<panes.length; i++)
		  panes[i].speichern(out);
  }
  
}
