/*

This file is part of jHaushalt.

jHaushalt is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

jHaushalt is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with jHaushalt; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

(C)opyright 2002-2005 Dr. Lars H. Hahn

*/

package haushalt.auswertung;

import haushalt.auswertung.bloecke.AbstractBlock;
import haushalt.auswertung.bloecke.BalkenBlock;
import haushalt.auswertung.bloecke.LeererBlock;
import haushalt.auswertung.bloecke.TextBlock;
import haushalt.daten.Datenbasis;
import haushalt.daten.Euro;
import haushalt.daten.zeitraum.AbstractZeitraum;
import haushalt.daten.zeitraum.Jahr;
import haushalt.gui.Haushalt;
import haushalt.gui.generischerdialog.AbstractGDPane;
import haushalt.gui.generischerdialog.EinOderAlleRegisterGDP;
import haushalt.gui.generischerdialog.ZeitraumGDP;
import haushalt.gui.generischerdialog.ZahlGDP;

import java.awt.Font;

/**
 * @author Dr. Lars H. Hahn
 * @version 2.0/2004.08.22
 */

public class BABalkenDiagramm extends AbstractBlockAuswertung {
	
	public static final String ueberschrift = "Balken-Diagramm Einahmen/Ausgaben";

  public BABalkenDiagramm(Haushalt haushalt, Datenbasis db, String name) {
    super(haushalt, db, name);
    AbstractGDPane[] panes = new AbstractGDPane[3];
		panes[0] = new ZeitraumGDP("Erster Zeitraum:", new Jahr(2004));
		panes[1] = new ZahlGDP("Anzahl Zeitrume:", new Integer(4));
		panes[2] = new EinOderAlleRegisterGDP("Register:", db.getRegisterNamen(), null);
		erzeugeEigenschaften(haushalt.getFrame(), ueberschrift, panes);
  }
 
  protected String berechneAuswertung(Object[] werte) {
  	AbstractZeitraum zeitraum = (AbstractZeitraum) werte[0];
  	int anzahlZeitraeume = ((Integer) werte[1]).intValue();
    String register = (String) werte[2];

    AbstractZeitraum tmpZeitraum = zeitraum;
    String[] zeitraumNamen = new String[anzahlZeitraeume];
    Euro[] einnahmen = new Euro[anzahlZeitraeume];
    Euro[] ausgaben = new Euro[anzahlZeitraeume];
		for(int i=0; i<anzahlZeitraeume; i++) {
      zeitraumNamen[i] = ""+tmpZeitraum;
      einnahmen[i] = db.getEinnahmen(tmpZeitraum, register);
      ausgaben[i] = db.getAusgaben(tmpZeitraum, register);
      tmpZeitraum = tmpZeitraum.folgeZeitraum();
		}		 

		// Vorhandene Blcke lschen und neu berechnete einfgen
		String titel = "Einnahmen/Ausgaben ("+zeitraumNamen[0]+" bis "+zeitraumNamen[anzahlZeitraeume-1];
		if(register == null)
		  titel += ")";
		else
		  titel += ", " + register + ")";
		loescheBloecke();
		AbstractBlock block1 = new TextBlock(titel);
		block1.setFont(new Font(haushalt.getFontname(), Font.BOLD, haushalt.getFontgroesse()+6));
		addDokumentenBlock(block1);
		addDokumentenBlock(new LeererBlock(1));
		AbstractBlock block2 = new BalkenBlock(zeitraumNamen, einnahmen, ausgaben);
		block2.setFont(new Font(haushalt.getFontname(), Font.PLAIN, haushalt.getFontgroesse()));
		addDokumentenBlock(block2);
    return titel;
	}

}
