/*

This file is part of jHaushalt.

jHaushalt is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

jHaushalt is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with jHaushalt; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

(C)opyright 2002-2005 Dr. Lars H. Hahn

*/

package haushalt.auswertung;

import haushalt.auswertung.bloecke.AbstractBlock;
import haushalt.auswertung.bloecke.LeererBlock;
import haushalt.auswertung.bloecke.TabellenBlock;
import haushalt.auswertung.bloecke.TextBlock;
import haushalt.auswertung.bloecke.TortenBlock;
import haushalt.daten.Datenbasis;
import haushalt.daten.Euro;
import haushalt.daten.Kategorie;
import haushalt.daten.zeitraum.AbstractZeitraum;
import haushalt.daten.zeitraum.Jahr;
import haushalt.gui.Haushalt;
import haushalt.gui.generischerdialog.AbstractGDPane;
import haushalt.gui.generischerdialog.EinOderAlleRegisterGDP;
import haushalt.gui.generischerdialog.TextArrayGDP;
import haushalt.gui.generischerdialog.ZahlGDP;
import haushalt.gui.generischerdialog.ZeitraumGDP;

import java.awt.Font;

/**
 * @author Dr. Lars H. Hahn
 * @version 2.0/2004.08.22
 */
public class BATortenDiagramm extends AbstractBlockAuswertung {
  
	public static final String ueberschrift = "Torten-Diagramm Einnahmen/Ausgaben";
  
  public BATortenDiagramm(Haushalt haushalt, Datenbasis db, String name) {
    super(haushalt, db, name);
    AbstractGDPane[] panes = new AbstractGDPane[4];
		panes[0] = new ZeitraumGDP("Erster Zeitraum:", new Jahr(2004));
		panes[1] = new ZahlGDP("Anzahl der Kategorien:", new Integer(10));
		panes[2] = new EinOderAlleRegisterGDP("Register:", db.getRegisterNamen(), null);
    panes[3] = new TextArrayGDP("Farbschema:", FarbPaletten.palettenNamen, "Standard");
		erzeugeEigenschaften(haushalt.getFrame(), ueberschrift, panes);
  }

  protected String berechneAuswertung(Object[] werte) {
  	AbstractZeitraum zeitraum = (AbstractZeitraum) werte[0];
  	int maxAnzahlWerte = ((Integer) werte[1]).intValue();
    String register = (String) werte[2];
    String farbschema = (String) werte[3];
    
    Euro[] einnahmen = new Euro[maxAnzahlWerte];
    Kategorie[] kategorienEinnahmen = new Kategorie[maxAnzahlWerte];
    Euro[] ausgaben = new Euro[maxAnzahlWerte];
    Kategorie[] kategorienAusgaben = new Kategorie[maxAnzahlWerte];
    Euro gesamtEinnahmen = new Euro();
    Euro gesamtAusgaben = new Euro();
    Kategorie[] kategorien = db.getKategorien();
    int anzahl = kategorien.length;
    Euro kategorieSumme;
    
    for(int i=0; i<anzahl; i++) {
      kategorieSumme = db.getKategorieSaldo(kategorien[i], zeitraum, register);
      if(kategorieSumme.compareTo(Euro.NULL_EURO)>0) { // Einnahmen:
        gesamtEinnahmen.sum(kategorieSumme);
        for(int j=0; j<maxAnzahlWerte; j++) {
          if(einnahmen[j] == null) {
            einnahmen[j] = kategorieSumme; // Prima! Noch ein Platz frei!
            kategorienEinnahmen[j] = kategorien[i];
            j = maxAnzahlWerte;
          }
          else if(kategorieSumme.compareTo(einnahmen[j]) > 0) {
            for(int k=maxAnzahlWerte-1; k>j; k--) {
              einnahmen[k] = einnahmen[k-1];
              kategorienEinnahmen[k] = kategorienEinnahmen[k-1];
            }
            einnahmen[j] = kategorieSumme;
            kategorienEinnahmen[j] = kategorien[i];
            j = maxAnzahlWerte;
          }
        }
      }
      else if(kategorieSumme.compareTo(Euro.NULL_EURO) < 0) { // Ausgaben:
        kategorieSumme = Euro.NULL_EURO.sub(kategorieSumme);
        gesamtAusgaben.sum(kategorieSumme);
        for(int j=0; j<maxAnzahlWerte; j++) {
          if(ausgaben[j] == null) {
            ausgaben[j] = kategorieSumme; // Prima! Noch ein Platz frei!
            kategorienAusgaben[j] = kategorien[i];
            j = maxAnzahlWerte;
          }
          else if(kategorieSumme.compareTo(ausgaben[j]) > 0) {
            for(int k=maxAnzahlWerte-1; k>j; k--) {
              ausgaben[k] = ausgaben[k-1];
              kategorienAusgaben[k] = kategorienAusgaben[k-1];
            }
            ausgaben[j] = kategorieSumme;
            kategorienAusgaben[j] = kategorien[i];
            j = maxAnzahlWerte;
          }
        }
      }
    }

		// Vorhandene Blcke lschen und neu berechnete einfgen
    String titel = "Verteilung Einnahmen/Ausgaben ("+zeitraum;
		loescheBloecke();
    if(register == null)
      titel += ")";
    else
      titel += ", "+register+")";
    AbstractBlock block1 = new TextBlock(titel);
		block1.setFont(new Font(haushalt.getFontname(), Font.BOLD, haushalt.getFontgroesse()+6));
    addDokumentenBlock(block1);
    addDokumentenBlock(new LeererBlock(1));
    AbstractBlock block2 = new TortenBlock(
      farbschema, einnahmen, gesamtEinnahmen, ausgaben, gesamtAusgaben
    );
		block2.setFont(new Font(haushalt.getFontname(), Font.PLAIN, haushalt.getFontgroesse()));
    addDokumentenBlock(block2);
    addDokumentenBlock(new LeererBlock(1));

    tabelle = new String[maxAnzahlWerte][4];
    for(int j=0; j<maxAnzahlWerte; j++) {
      if(einnahmen[j] != null) {
        tabelle[j][0] = ""+kategorienEinnahmen[j];
        tabelle[j][1] = ""+einnahmen[j];
      }
      else {
        tabelle[j][0] = "";
        tabelle[j][1] = "";
      }
      if(ausgaben[j] != null) {
        tabelle[j][2] = ""+kategorienAusgaben[j];
        tabelle[j][3] = ""+ausgaben[j];
      }
      else {
        tabelle[j][2] = "";
        tabelle[j][3] = "";
      }
    }
    final double[] relTabs = {0.0, 25.0, 50.0, 75.0};
    final int[] attribute = {TabellenBlock.LINKS, TabellenBlock.RECHTS, TabellenBlock.LINKS, TabellenBlock.RECHTS};
    TabellenBlock block3 = new TabellenBlock(tabelle);
		block3.setFont(new Font(haushalt.getFontname(), Font.PLAIN, haushalt.getFontgroesse()));
    block3.setRelTabs(relTabs);
    block3.setAusrichtung(attribute);
    block3.setHgFarbe(farbschema);
    block3.setLinienFarbe("Wei");
    block3.setRelRand(0.05D);
    addDokumentenBlock(block3);
    return titel;
  }

}
