/*

This file is part of jHaushalt.

jHaushalt is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

jHaushalt is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with jHaushalt; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

(C)opyright 2002-2005 Dr. Lars H. Hahn

*/

package haushalt.auswertung;

import haushalt.daten.Datenbasis;
import haushalt.gui.Haushalt;

import java.awt.BorderLayout;
import java.awt.Container;
import java.awt.Cursor;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.print.PageFormat;
import java.awt.print.PrinterException;
import java.awt.print.PrinterJob;
import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.ArrayList;

import javax.swing.DefaultListModel;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JList;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSplitPane;
import javax.swing.ListSelectionModel;
import javax.swing.ScrollPaneConstants;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

/**
 * @author Dr. Lars H. Hahn
 * @version 2.0/2004.08.22
 */

public class ContainerAuswertung extends JDialog implements ListSelectionListener {
  private final boolean DEBUG = false;

	private final Haushalt haushalt;
  private final Datenbasis db;
  private final PageFormat seitenFormat;
  private final ArrayList auswertungen = new ArrayList();
	
	// Liste und Ausgabefenster:
  private final DefaultListModel listModel = new DefaultListModel();
	private final JList list = new JList(listModel);
	private final JScrollPane listScrollPane = new JScrollPane(list);
	private final JScrollPane graphikScrollPane = new JScrollPane();
	private final JSplitPane split= new JSplitPane(JSplitPane.HORIZONTAL_SPLIT, listScrollPane, graphikScrollPane);
  private final Dimension minimumSize = new Dimension(100, 50);

	// ButtonPane und Buttons:
  private final JPanel paneButton = new JPanel();
	private final JButton buttonHinzu;
	private final JButton buttonEntf;
	private final JButton buttonEigensch;
  private final JButton buttonExport;
	private final JButton buttonDrucken;
	private final JButton buttonAbbruch;

  private final AuswertungAuswaehlenDlg dlg;
  
  public ContainerAuswertung(Haushalt haushalt, Datenbasis db, PageFormat seitenFormat) {
    super(haushalt.getFrame(), "Auswertungen", true);
    this.haushalt = haushalt;
    this.db = db;
    this.seitenFormat = seitenFormat;

    buttonHinzu = new JButton("Hinzufgen", haushalt.bildLaden("Add16.gif"));
  	buttonEntf = new JButton("Entfernen", haushalt.bildLaden("Delete16.gif"));
  	buttonEigensch = new JButton("Eigenschaften", haushalt.bildLaden("Properties16.gif"));
    buttonExport = new JButton("CSV-Export", haushalt.bildLaden("Export16.gif"));
  	buttonDrucken = new JButton("Drucken", haushalt.bildLaden("Print16.gif"));
  	buttonAbbruch = new JButton("Schlieen");
    
		// Liste zur Auswahl der Auswertung
		list.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		list.addListSelectionListener(this);
		
    // SplitPane enthlt links die Liste der Auswertungen und
    // rechts das Fenster der gewhlten Auswertung
    split.setOneTouchExpandable(true);
		split.setDividerLocation(150);
		listScrollPane.setMinimumSize(minimumSize);
		graphikScrollPane.setMinimumSize(minimumSize);
		graphikScrollPane.setPreferredSize(new Dimension(500, 500));
		graphikScrollPane.setVerticalScrollBarPolicy(ScrollPaneConstants.VERTICAL_SCROLLBAR_ALWAYS);

    // Buttons zur ButtonPane hinzufgen:
		paneButton.add(buttonHinzu, null);
		paneButton.add(buttonEntf, null);
		paneButton.add(buttonEigensch, null);
		paneButton.add(buttonExport, null);
		paneButton.add(buttonDrucken, null);
		paneButton.add(buttonAbbruch, null);
		buttonEntf.setEnabled(false);
		buttonEigensch.setEnabled(false);
		buttonExport.setEnabled(false);
		buttonDrucken.setEnabled(false);
		buttonHinzu.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				hinzufuegen();
			}
		});
		buttonEntf.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				entfernen();
			}
		});
		buttonEigensch.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				eigenschaften();
			}
		});
		buttonExport.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				export();
			}
		});
		buttonDrucken.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				drucken();
			}
		});
		buttonAbbruch.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				setVisible(false);
			}
		});

    Container contentPane = getContentPane();
    contentPane.add(split, BorderLayout.CENTER);
    contentPane.add(paneButton, BorderLayout.SOUTH);
		getRootPane().setDefaultButton(buttonEigensch);
		
		// Auswahl-Dialog erzeugen 
		dlg = new AuswertungAuswaehlenDlg(haushalt, db);
  }
  
  public void zeigeDialog() {
    final int anzahl = auswertungen.size();
    if(anzahl > 0) {
      haushalt.getFrame().setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
			for(int i=0; i<anzahl; i++) {
				AbstractAuswertung auswertung = (AbstractAuswertung)auswertungen.get(i);
				auswertung.berechneAuswertung();
			}
      list.setSelectedIndex(0);
      haushalt.getFrame().setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
    }
    pack();
    setVisible(true);
  }

  protected void hinzufuegen() {
    AbstractAuswertung auswertung = dlg.showDialog();
    if((auswertung != null) && auswertung.zeigeEigenschaften()) {
    	auswertungen.add(auswertung);
    	listModel.addElement(auswertung);
    	list.setSelectedIndex(listModel.getSize()-1);
			graphikScrollPane.getViewport().getView().repaint();
    }
  }
  
  protected void entfernen() {
		final int nr = list.getSelectedIndex();
		final int anzahl = auswertungen.size();
		if((nr == -1) || (nr >= anzahl))
			JOptionPane.showMessageDialog(null,
				"Keine Auswertung selektiert!",
				"Auswertungen",
				JOptionPane.WARNING_MESSAGE);
		else {
			auswertungen.remove(nr);
			listModel.remove(nr);
			if(anzahl > 1)
			  list.setSelectedIndex(0);
		}
	}

  protected void eigenschaften() {
		int nr = list.getSelectedIndex();
		if((nr == -1) || (nr >= auswertungen.size())){
			JOptionPane.showMessageDialog(null,
				"Keine Auswertung selektiert!",
				"Auswertungen",
				JOptionPane.WARNING_MESSAGE);
		}
		else {
			AbstractAuswertung auswertung = (AbstractAuswertung)auswertungen.get(nr);
			auswertung.zeigeEigenschaften();
			list.repaint();
			graphikScrollPane.getViewport().getView().repaint();
		}
	}
	
  protected void export() {
		int nr = list.getSelectedIndex();
		if((nr == -1) || (nr >= auswertungen.size())){
			JOptionPane.showMessageDialog(null,
				"Keine Auswertung selektiert!",
				"Auswertungen",
				JOptionPane.WARNING_MESSAGE);
		}
		else {
			AbstractAuswertung auswertung = (AbstractAuswertung)auswertungen.get(nr);
	    String[][] tabelle = auswertung.getTabelle();
      CsvHandler handler = new CsvHandler(tabelle);
      handler.exportDlg(haushalt.getFrame(), haushalt.getOrdner());
		}
  }
  
  protected void drucken() { 
		int nr = list.getSelectedIndex();
		if((nr == -1) || (nr >= auswertungen.size())){
			JOptionPane.showMessageDialog(null,
				"Keine Auswertung selektiert!",
				"Auswertungen",
				JOptionPane.WARNING_MESSAGE);
		}
		else {
			AbstractAuswertung auswertung = (AbstractAuswertung)auswertungen.get(nr);
			PrinterJob job = PrinterJob.getPrinterJob();
			job.setJobName("jHaushalt - Auswertung");
			job.setPrintable(auswertung, seitenFormat);
			if(job.printDialog())
  			try {
          job.print();
        } catch (PrinterException e) {
          e.printStackTrace();
        }
		}
	}
	
	public void valueChanged(ListSelectionEvent e) {
		if (e.getValueIsAdjusting() == false) {
			int nr = list.getSelectedIndex();
			if (nr == -1) {
				//Nichts selektiert, disable Eigenschaften + Entfernen
				buttonEntf.setEnabled(false);
				buttonEigensch.setEnabled(false);
				buttonExport.setEnabled(false);
				buttonDrucken.setEnabled(false);
			}
			else {
				buttonEntf.setEnabled(true);
				buttonEigensch.setEnabled(true);
				buttonDrucken.setEnabled(true);
				buttonExport.setEnabled(false);
				if(nr < auswertungen.size()) {
				  AbstractAuswertung auswertung = (AbstractAuswertung)auswertungen.get(nr);
				  if(auswertung.getTabelle() != null)
				    buttonExport.setEnabled(true);
				  graphikScrollPane.getViewport().setView(auswertung);
				}
			}
		}
	}
	
	public void laden(String dateiname) {
    File datei = new File(dateiname);
    if(datei.exists())
      try {
		    FileInputStream fis = new FileInputStream(datei);
		    DataInputStream in = new DataInputStream(fis);
		    int size = in.readInt();
		    for(int i=0; i<size; i++) {
		      String name = in.readUTF();
		      String klasse = in.readUTF();
		      AbstractAuswertung auswertung = AbstractAuswertung.erzeugeAuswertung(klasse, haushalt, db, name);
		      auswertung.laden(in);
		    	auswertungen.add(auswertung);
		    	listModel.addElement(auswertung);
		    }
		    fis.close();
		    if(DEBUG)
		      System.out.println(""+size+" Auswertungen geladen.");
      }
    	catch(IOException e) {
    	  e.printStackTrace();
    	}
	}
	
	public void speichern(String dateiname) {
    File datei = new File(dateiname);
		try {
      FileOutputStream fos = new FileOutputStream(datei);
      DataOutputStream out = new DataOutputStream(fos);
      out.writeInt(auswertungen.size());
			for(int i=0; i<auswertungen.size(); i++) {
				AbstractAuswertung auswertung = (AbstractAuswertung)auswertungen.get(i);
        out.writeUTF(""+auswertung);
        out.writeUTF(auswertung.getClass().getName());
				auswertung.speichern(out);
			}
      out.flush();
      fos.close();
      if(DEBUG)
        System.out.println(""+auswertungen.size()+" Auswertungen gespeichert.");
		}
		catch(IOException e) {
		  e.printStackTrace();
		}
	}

}
