/*

This file is part of jHaushalt.

jHaushalt is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

jHaushalt is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with jHaushalt; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

(C)opyright 2002-2005 Dr. Lars H. Hahn

*/

package haushalt.auswertung;

import haushalt.gui.DeleteableTextField;

import java.awt.BorderLayout;
import java.awt.Container;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.ArrayList;

import javax.swing.Box;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;

/**
 * @author Dr. Lars H. Hahn
 * @version 2.01/2004.08.25
 */

 /*
  * 2004.08.25 BugFix: OK / Abruch des Im-/Export-Dialogs weitergegeben.
  */

public class CsvHandler {
  private final boolean DEBUG = false;

  private Object[][] tabelle = {{"Leer"}};
  protected char datensatzTeiler = ';';

  public CsvHandler() {
    // OK
  }
  
  public CsvHandler(Object[][] tabelle) {
    this.tabelle = tabelle;
    if(DEBUG)
      System.out.println("CsvHandler ["+tabelle.length+"]["+tabelle[0].length+"]");
  }

  public CsvHandler(DataInputStream in) {
    try {
      read(in);
    }
    catch(IOException e) {
      e.printStackTrace();
    }
  }

  private String trimZelle(String zelle) {
    if(zelle.equals("\"\""))
      zelle = "";
    else if(zelle.startsWith("\"") && zelle.endsWith("\""))
      zelle = zelle.substring(1, zelle.length()-1);
    if(DEBUG)
      System.out.println(zelle);
    return zelle;
  }

  public String[][] getTabelle() {
    int ya = tabelle.length;
    int xa = tabelle[0].length;
    String[][] t = new String[ya][xa];
    for(int y=0; y<ya; y++)
      for(int x=0; x<xa; x++)
        t[y][x] = tabelle[y][x].toString();
    return t;
  }

  protected void read(DataInputStream in)
    throws IOException, FileNotFoundException {
    ArrayList neueZeile = new ArrayList();
    ArrayList zeilen = new ArrayList();
    String zelle = "";
    int z;
    int zellenProZeile = 0;
    while((z = in.read()) != -1) {
      char c = (char)z;
      if((z == 13) || (z == 10)) {
        if(neueZeile.size() > 0) {
          neueZeile.add(trimZelle(zelle));
          zelle = "";
          zeilen.add(neueZeile);
          if(neueZeile.size() > zellenProZeile)
            zellenProZeile = neueZeile.size();
          neueZeile = new ArrayList();
        }
      }
      else if(c == datensatzTeiler) {
        if(DEBUG)
          System.out.print(zelle+", ");
        neueZeile.add(trimZelle(zelle));
        zelle = "";
      }
      else
        zelle += c;
    }
    tabelle = new Object[zeilen.size()][zellenProZeile];
    for(int y=0; y<zeilen.size(); y++) {
      tabelle[y] = ((ArrayList)zeilen.get(y)).toArray(tabelle[y]);
    }
  }

  protected void write(DataOutputStream out)
    throws IOException {
    for(int y=0; y<tabelle.length; y++) {
      for(int x=0; x<tabelle[y].length; x++) {
        if(x>0)
          out.writeByte(datensatzTeiler);
        out.writeBytes("\""+tabelle[y][x]+"\"");
        if(DEBUG)
          System.out.print(" "+tabelle[y][x]);
      }
      if(DEBUG)
        System.out.println("");
      out.writeByte(13);
      out.writeByte(10);
    }
  }

  public void setDatensatzTeiler(char datensatzTeiler) {
    this.datensatzTeiler = datensatzTeiler;
  }

  public boolean importDlg(JFrame frame, String path) {
    CsvDateiDialog dlg = new CsvDateiDialog(frame, path, true);
    dlg.pack();
    dlg.setVisible(true);
    return dlg.ok;
  }

  public boolean exportDlg(JFrame frame, String path) {
    CsvDateiDialog dlg = new CsvDateiDialog(frame, path, false);
    dlg.pack();
    dlg.setVisible(true);
    return dlg.ok;
  }

  private class CsvDateiDialog extends JDialog {

    private final JPanel hauptPane = new JPanel();
    private final JPanel buttonPane = new JPanel();
    protected final DeleteableTextField dateiname = new DeleteableTextField(20);
    protected final DeleteableTextField trennzeichen = new DeleteableTextField(""+datensatzTeiler, 2);
    private final JButton buttonAuswahl = new JButton("Auswahl");
    private final JButton buttonOK = new JButton("OK");
    private final JButton buttonAbbruch = new JButton("Abbruch");
    protected final JFrame frame;
    protected final String path;
    protected final boolean laden;
    protected boolean ok = true;

    private CsvDateiDialog(JFrame f, String p, boolean l) {
      super(f, "CSV Im-/Export", true);
      this.frame = f;
      this.laden = l;
      this.path = p;

      Container contentPane = getContentPane();
      contentPane.add(hauptPane, BorderLayout.CENTER);
      contentPane.add(buttonPane, BorderLayout.SOUTH);
      hauptPane.setLayout(new GridLayout(0,2));
      hauptPane.add(new JLabel("Dateiname:"));
      hauptPane.add(dateiname);
      hauptPane.add(Box.createGlue());
      hauptPane.add(buttonAuswahl);
      hauptPane.add(new JLabel("Trennzeichen:"));
      hauptPane.add(trennzeichen);
      buttonPane.add(buttonOK);
      buttonPane.add(buttonAbbruch);

      buttonAuswahl.addActionListener(new ActionListener() {
        public void actionPerformed(ActionEvent e) {
          JFileChooser dateidialog  = new JFileChooser(path);
          if(laden) {
            if(dateidialog.showOpenDialog(frame) == JFileChooser.APPROVE_OPTION) {
              dateiname.setText(dateidialog.getSelectedFile().getAbsolutePath());
            }
          }
          else {
            if(dateidialog.showSaveDialog(frame) == JFileChooser.APPROVE_OPTION) {
              String text = dateidialog.getSelectedFile().getAbsolutePath();
              if(!text.toLowerCase().endsWith(".csv"))
                text += ".csv";
              dateiname.setText(text);
            }
          }
        }
      });
      buttonOK.addActionListener(new ActionListener() {
        public void actionPerformed(ActionEvent e) {
          datensatzTeiler = trennzeichen.getText().toCharArray()[0];
          try {
            if(laden) {
              FileInputStream fis = new FileInputStream(dateiname.getText());
              DataInputStream in = new DataInputStream(fis);
              read(in);
              fis.close();
            }
            else {
              FileOutputStream fos = new FileOutputStream(dateiname.getText());
              DataOutputStream out = new DataOutputStream(fos);
              write(out);
              out.flush();
              fos.close();
            }
            ok = true;
          }
          catch(IOException ex) {
            ex.printStackTrace();
            ok = false;
          }
          setVisible(false);
        }
      });
      buttonAbbruch.addActionListener(new ActionListener() {
        public void actionPerformed(ActionEvent e) {
          ok = false;
          setVisible(false);
        }
      });
      getRootPane().setDefaultButton(buttonAbbruch);
    }

  }

  public static void main(String[] args)
   throws Exception {
    FileInputStream fis = new FileInputStream("test-out.csv");
    DataInputStream in = new DataInputStream(fis);
    CsvHandler handler = new CsvHandler(in);
    fis.close();

    FileOutputStream fos = new FileOutputStream("test-out2.csv");
    DataOutputStream out = new DataOutputStream(fos);
    handler.write(out);
    out.flush();
    fos.close();
  }
}
