/*
 
 This file is part of jHaushalt.
 
 jHaushalt is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.
 
 jHaushalt is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with jHaushalt; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
 (C)opyright 2002-2005 Dr. Lars H. Hahn
 
 */

package haushalt.gui;
import haushalt.auswertung.*;
import haushalt.daten.*;
import haushalt.daten.Datenbasis.QuickenImportException;
import haushalt.gui.dialoge.*;
import haushalt.gui.generischerdialog.*;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.Image;
import java.awt.Point;
import java.awt.event.*;
import java.awt.print.*;
import java.io.*;
import java.lang.reflect.Field;
import java.net.URL;
import java.net.URLClassLoader;
import java.util.*;

import javax.swing.*;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import javax.swing.filechooser.FileFilter;
import javax.swing.table.DefaultTableCellRenderer;
import javax.swing.table.DefaultTableColumnModel;
import javax.swing.table.TableColumnModel;

/**
 * @author Dr. Lars H. Hahn
 * @version 2.02/2005.04.30
 */

 /* 2005.04.30 BugFix: Nach Splitten Buchung merken
  * 2005.03.10 Erweiterung: Gemerkte Buchungen ab Datum
  * 2005.02.18 BugFix: Nachdem Splitten einer Buchung wurde nicht 
  * "mitgesprungen". 
  * 2004.08.25 BugFix: Abbruch beim CSV-Import bercksichtigt.
  */
public class Haushalt implements KeyListener, ListSelectionListener {
  private final boolean DEBUG = false;
  
  private final String copyright = "jHaushalt v2.02 * (C)opyright 2002-2005 Lars H. Hahn";
  private final String version = "2.02";
  
  private final Properties properties;
  private final JFrame frame = new JFrame();
  private final JTabbedPane tabbedPane;
  private TableColumnModel columnModel = null;
  private final Image icon;
  private final JLabel hintergrund;
  protected final JTextField status = new JTextField(copyright);
  private final GemerkteBuchungenGlassPane glassPane = new GemerkteBuchungenGlassPane();
  private final DlgOptionen dlgOptionen;
  private final DlgSuchenErsetzen dlgSuchenErsetzen;
  private PageFormat seitenFormat;
  private Datenbasis db;
  private ContainerAuswertung containerAuswertung;
  
  /**
   * Einziger Konstruktor.
   */
  public Haushalt(String dateiname) {
    // Properties laden
    properties = new Properties();
    if(DEBUG)
      System.out.println("Lade die Properties.");
    String userHome = System.getProperty("user.home");
    properties.setProperty("jhh.ordner", userHome); // Default-Wert Arbeitsordner
    File datei = new File(userHome,".jhh");
    if(datei.exists()) {
      try {
        FileInputStream fis = new FileInputStream(datei);
        properties.load(fis);
        fis.close();
        if(DEBUG)
          properties.list(System.out);
      }
      catch(Exception e) {
        if(DEBUG)
          System.out.println("-W- Keine Properties geladen.");
      }
    }
    
    // Erzeuge Hauptfenster
    Container contentPane = frame.getContentPane();
    if(DEBUG)
      System.out.println("Hauptfenster: Lade das Icon.");
    icon = bildLaden("jhh-icon.gif").getImage();
    frame.setIconImage(icon);
    if(DEBUG)
      System.out.println("Hauptfenster: Lade den Hintergrund.");
    hintergrund = new JLabel(bildLaden("jhh-hintergrund.gif"));
    contentPane.add(hintergrund, BorderLayout.CENTER);
    tabbedPane = new JTabbedPane(SwingConstants.BOTTOM);
    int breite = new Integer(properties.getProperty("jhh.register.breite", "600")).intValue();
    int hoehe = new Integer(properties.getProperty("jhh.register.hoehe", "400")).intValue();
    tabbedPane.setPreferredSize(new Dimension(breite, hoehe));
    frame.setGlassPane(glassPane);
    frame.setTitle(copyright);
    // Das Men wird initialisiert
    if(DEBUG)
      System.out.println("Initialisiere das Men.");
    ActionHandler actionHandler = new ActionHandler(this);
    frame.setJMenuBar(actionHandler.erzeugeMenuBar());
    contentPane.add(actionHandler.erzeugeToolBar(), BorderLayout.PAGE_START);
    status.setEditable(false);
    contentPane.add(status, BorderLayout.PAGE_END);
    frame.setDefaultCloseOperation(WindowConstants.DO_NOTHING_ON_CLOSE);
    frame.addWindowListener(new WindowAdapter() {
      public void windowClosing(WindowEvent e) {
        beenden();
      }
    });
    
    seitenFormat = PrinterJob.getPrinterJob().defaultPage();
    // Letzte Datei laden oder neu initialisieren
    // In jedem Fall wird die Datenbasis erzeugt.
    if(dateiname != null)
      laden(new File(dateiname));
    else if(properties.containsKey("jhh.dateiname"))
      laden(new File(properties.getProperty("jhh.dateiname")));
    else {
      db = new Datenbasis();
      containerAuswertung = new ContainerAuswertung(this, db, seitenFormat);
    }
    
    // Dialog fr die Optionen erzeugen; die Optionen sind eine
    // Teilmenge der Properties 
    if(DEBUG)
      System.out.println("Options-Dialog initialisieren.");
    dlgOptionen = new DlgOptionen(frame, properties);
    
    // Dialog Suchen/ersetzen erzeugen
    dlgSuchenErsetzen = new DlgSuchenErsetzen(this);
    
    oberflaecheAnpassen(); // hier werden auch andere Optionen gesetzt
    
    if(DEBUG)
      System.out.println("Applikation initialisiert.");
  }
  
  /**
   * Liefert das Hauptfenster.
   * @return Hauptfenster
   */
  public JFrame getFrame() {
    return frame;
  }
  
  /**
   * Liefert das Arbeitsverzeichnis.
   * @return Arbeitsverzeichnis
   */
  public String getOrdner() {
    return properties.getProperty("jhh.ordner");
  }
  
  // -- Einstellungen ---------------------------------------------------------
  
  /**
   * bernimmt die optionalen Einstellungen des Benutzters.
   *
   */
  private void oberflaecheAnpassen() {
    Euro.setWaehrungssymbol(properties.getProperty("jhh.opt.waehrung", ""));
    setTabPlacement(properties.getProperty("jhh.opt.reiter", "BOTTOM"));
    for(int i=0; i<tabbedPane.getTabCount(); i++) {
      JScrollPane scrollPane = (JScrollPane) tabbedPane.getComponent(i);
      JTable table = (JTable) scrollPane.getViewport().getView();
      table.setSelectionBackground(getFarbeSelektion());
      table.setGridColor(getFarbeGitter());
    }
    db.setUnterkategorienVerwenden(Boolean.valueOf(properties.getProperty("jhh.opt.unterkat", "true")).booleanValue());
    db.setStartDatum(new Datum(properties.getProperty("jhh.opt.startdatum", "01.01.00")));
    String text = properties.getProperty("jhh.opt.deltaste", "Entf");
    int deltaste;
    if(text.equals("Shift-Entf"))
      deltaste = InputEvent.SHIFT_MASK;
    else if(text.equals("Strg-Entf"))
      deltaste = InputEvent.CTRL_MASK;
    else if(text.equals("Alt-Entf"))
      deltaste = InputEvent.ALT_MASK;
    else
      deltaste = 0;
    DeleteableTextField.setDeltaste(deltaste);
  }
  
  public Color getFarbeSelektion() {
    int farbe = new Integer(properties.getProperty("jhh.opt.selektion", "12632256")).intValue(); // #c0c0c0 
    return new Color(farbe);
  }
  
  public Color getFarbeGitter() {
    int farbe = new Integer(properties.getProperty("jhh.opt.gitter", "10066329")).intValue(); // #999999 
    return new Color(farbe);
  }
  
  public String getFontname() {
    return properties.getProperty("jhh.opt.font", "SansSerif");
  }
  
  public int getFontgroesse() {
    return new Integer(properties.getProperty("jhh.opt.punkt", "12")).intValue();
  }
  
  /**
   * Ldt Bilder aus dem Verzeichnis <code>res/</code>.
   * @param dateiname Dateiname des Bildes (ohne Pfad)
   */
  public ImageIcon bildLaden(String dateiname) {
    URLClassLoader urlLoader = (URLClassLoader)getClass().getClassLoader();
    URL fileLoc = urlLoader.findResource("res/" + dateiname);
    if(DEBUG)
      System.out.println("Lade Bild @ "+fileLoc);
    return new ImageIcon(fileLoc);
  }
  
  /**
   * Setzt den Ort der Register-Reiter.
   * @param tabPlacement "TOP", "BOTTOM", "LEFT" oder "RIGHT"
   */
  private void setTabPlacement(String tabPlacement) { 
    int tp = SwingConstants.BOTTOM;
    Class c = SwingConstants.class;
    try {
      Field field = c.getField(tabPlacement);
      tp = field.getInt(new Integer(0));
    }
    catch(NoSuchFieldException e) {
      e.printStackTrace();
    }
    catch(IllegalAccessException e) {
      e.printStackTrace();
    }
    tabbedPane.setTabPlacement(tp);
  }
  
  // -- Register-Tabellen ------------------------------------------------------
  
  /**
   * Liefert den Index einer Register-Tabelle.
   * @param name Name des Registers
   */
  private int getTabNr(String name) {
    if(tabbedPane != null)
      for(int i=0; i<tabbedPane.getTabCount(); i++)
        if(name.equals(tabbedPane.getTitleAt(i)))
          return i;
    return -1;
  }
  
  /**
   * Wandelt alle Register in Tabellen um und zeigt sie so an.
   */
  private void zeigeAlleRegister() {
    String[] register = db.getRegisterNamen();
    for(int i=0; i<register.length; i++)
      zeigeRegister(register[i]);
  }
  
  /**
   * Wandelt ein Register in eine Tabelle um und zeigt sie so an.
   * @param regname Name des Registers
   */
  private void zeigeRegister(String regname) {
    RegisterTableModel tableModel = new RegisterTableModel(this, db, regname);
    int tabNr = getTabNr(""+tableModel);
    if(tabNr > -1) {
      // Register wird schon angezeigt, wahrscheinlich wurde es verndert:
      registerVeraendert(tabNr);
      return;
    }
    JTable table;
    if(columnModel == null) {
      // Das ColumnModel wird von der ersten erzeugten Tabelle genommen und
      // initialisiert. Alle Tabellen erhalten so die gleichen Spalten-Breiten.
      table = new JTable(tableModel);
      columnModel = table.getColumnModel();
      if(properties.containsKey("jhh.register.spalte0"))
        for(int j=0; j<columnModel.getColumnCount(); j++) {
          int breite = new Integer(properties.getProperty("jhh.register.spalte"+j)).intValue();
          columnModel.getColumn(j).setPreferredWidth(breite);
          if(DEBUG)
            System.out.println(""+tableModel+": Breite Spalte "+j+" = "+breite);
        }
    }
    else
      table = new JTable(tableModel, columnModel);
    table.setSurrendersFocusOnKeystroke(true);
    table.getSelectionModel().addListSelectionListener(this);
    table.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
    table.setSelectionBackground(getFarbeSelektion());
    table.setGridColor(getFarbeGitter());
    
    // Cell-Editoren erzeugen
    table.setDefaultEditor(Datum.class, new DefaultCellEditor(new DatumField()));
    DeleteableTextField textField = new DeleteableTextField();
    textField.addKeyListener(this);
    DefaultCellEditor cellEditor = new DefaultCellEditor(textField);
    table.setDefaultEditor(String.class, cellEditor);
    table.setDefaultEditor(Object.class, new KategorieCellEditor(this, db));
    table.setDefaultEditor(Euro.class, new DefaultCellEditor(new EuroField()));
    // Cell-Renderer erzeugen
    table.setDefaultRenderer(Datum.class, new DefaultTableCellRenderer());
    table.setDefaultRenderer(String.class, new DefaultTableCellRenderer());
    table.setDefaultRenderer(Object.class, new KategorieRenderer());
    table.setDefaultRenderer(Euro.class, new EuroRenderer());
    
    JScrollPane scrollPane = new JScrollPane(table);
    scrollPane.setVerticalScrollBarPolicy(ScrollPaneConstants.VERTICAL_SCROLLBAR_ALWAYS);
    tabbedPane.add(scrollPane, ""+tableModel);
    table.setRowSelectionInterval(table.getRowCount()-1,table.getRowCount()-1);
    JScrollBar scrollBar = scrollPane.getVerticalScrollBar();
    // JavaBug: Hier ist der maxValue noch 100 ...
    scrollBar.setValue(100);
    // Hier hat MaxValue den tatsaechlichen Wert ...
    scrollBar.setValue(scrollBar.getMaximum());
    
    if(tabbedPane.getTabCount() == 1) {
      frame.getContentPane().remove(hintergrund);
      frame.getContentPane().add(tabbedPane, BorderLayout.CENTER);
      frame.validate();
      if(DEBUG)
        System.out.println("Register '"+regname+"' ist erste Table -> tabbedPane angezeigt.");
    }
    
    frame.repaint();
    if(DEBUG)
      System.out.println("Table zu Register '"+regname+"' erzeugt.");
  }
  
  /**
   * Lscht die Tabelle zu einem Register. Falls keine Tabelle mehr vorhanden ist,
   * wird wieder das Hintergrundbild angezeigt.
   * @param regname Name des Registers
   */
  private void entferneRegister(String regname) {
    for(int i=0; i<tabbedPane.getTabCount(); i++)
      if(regname.equals(tabbedPane.getTitleAt(i))) {
        JScrollPane scrollPane = (JScrollPane) tabbedPane.getComponentAt(i);
        JTable table = (JTable) scrollPane.getViewport().getView();
        // Die Register teilen sich ein ColumnModel, bevor ein Register gelscht wird
        // muss daher das ColumnModel neu gesetzt werden.
        table.setColumnModel(new DefaultTableColumnModel());
        tabbedPane.removeTabAt(i);
      }
    if(tabbedPane.getTabCount() == 0) {
      columnModel = null;
      frame.getContentPane().remove(tabbedPane);
      frame.getContentPane().add(hintergrund, BorderLayout.CENTER);
      frame.validate();
    }
    frame.repaint();
  }
  
  private void entferneAlleRegister() {
    Container contentPane = frame.getContentPane();
    tabbedPane.removeAll();
    contentPane.remove(tabbedPane);
    columnModel = null;
    contentPane.add(hintergrund, BorderLayout.CENTER);
    frame.validate();
    frame.repaint();
  }
  
  private void registerVeraendert(int nr) {
    JScrollPane scrollPane = (JScrollPane) tabbedPane.getComponentAt(nr);
    JTable table = (JTable) scrollPane.getViewport().getView();
    RegisterTableModel tableModel = (RegisterTableModel) table.getModel();
    tableModel.fireTableDataChanged();
  }
  
  public void alleRegisterVeraendert() {
    for(int i=0; i<tabbedPane.getTabCount(); i++)
      registerVeraendert(i);
  }
  
  /**
   * Whlt eine bestimmte Buchung in einer der Register-Tabellen aus.
   * @param regname Name des Registers
   * @param buchungIndex Zeile der Buchung
   */
  public void selektiereBuchung(String regname, int buchungIndex) {
    int tabIndex = getTabNr(regname);
    if(tabIndex > -1) {
      tabbedPane.setSelectedIndex(tabIndex);
      JScrollPane scrollPane = (JScrollPane)tabbedPane.getComponentAt(tabIndex);
      JViewport viewport = scrollPane.getViewport();
      JTable table = (JTable)viewport.getComponent(0);
      if(buchungIndex >= table.getRowCount())
        buchungIndex = table.getRowCount() - 1 ;
      table.setRowSelectionInterval(buchungIndex, buchungIndex);
      table.scrollRectToVisible(table.getCellRect(buchungIndex, 0, true));
      if(DEBUG)
        System.out.println("selektiereBuchung: Buchung Nr. "+buchungIndex+" in Register "+regname+" selektiert.");
    }
    else
      status.setText("-E- selektiereBuchung: Register "+regname+" nicht gefunden.");
  }
  
  // -- Gemerkte Buchungen -----------------------------------------------------
  
  /**
   * Setzt die Koordinaten an denen die gemerkte Buchung erscheinen soll.
   * Die Koordinaten werden in Abhngigkeit von dem TextField gesetzt in dem
   * der Buchungstext eingegeben wird.
   * @param comp Component = JTextField
   */
  public void setzteKoordinatenGemerkteBuchung(Component comp) {
    Point xy = SwingUtilities.convertPoint(comp.getParent(), comp.getX(), comp.getY(), glassPane);
    glassPane.setTextKoordinaten(xy);
  }
  
  public boolean gemerkteBuchungen() {
    return Boolean.valueOf(properties.getProperty("jhh.opt.gemerkte", "true")).booleanValue();
  }
  
  /**
   * Zeigt eine zuvor gemerkte Buchung an. 
   * @param prefix Anfang des Buchungstextes
   */
  public void zeigeGemerkteBuchung(String prefix) {
    if(gemerkteBuchungen()) {
      AbstractBuchung buchung = db.findeGemerkteBuchung(prefix);
      if(buchung != null) {
        String text = buchung.getText()+" / "+buchung.getKategorie()+" / "+buchung.getWert();
        glassPane.setText(text);
        glassPane.setVisible(true);
      }
      else
        glassPane.setVisible(false);	  	
    }
  }
  
  /**
   * berprft, ob schon ein Register angelegt wurde
   * @return <code>false</code> mindestens ein Register vorhanden
   */
  private boolean keinRegisterVorhanden() {
    if(tabbedPane.getTabCount() == 0) {
      int n = JOptionPane.showConfirmDialog(frame,
          "Es gibt noch kein Register.\nWollen Sie jetzt ein Register anlegen?",
          "Problem",
          JOptionPane.YES_NO_OPTION);
      if(n == JOptionPane.OK_OPTION)
        neuesRegister();
      return true;
    }
    return false;
  }
  
  // == Men: Datei ============================================================
  
  /**
   * berprft, ob die Applikationsdaten gendert wurden. Wird von neu() und 
   * beenden() aufgerufen.
   * @return <code>true</code> - Applikation wurde gendert,
   *         <code>false</code> - Daten sind gespeichert
   */
  private boolean abfrageGeaendert() {
    if(db.istGeaendert()) {
      int n = JOptionPane.showConfirmDialog(frame,
      "Die Daten wurden gendert.\nSoll zunchst gespeichert werden?");
      switch(n) {
      case JOptionPane.CANCEL_OPTION: return false;
      case JOptionPane.NO_OPTION: return true;
      case JOptionPane.OK_OPTION: speichern(); break;
      }
      if(db.istGeaendert()) // Speichern war nicht erfolgreich!
        return false;
    }
    return true;
  }
  
  public void neu() {
    if(abfrageGeaendert()) {
      properties.remove("jhh.dateiname");
      entferneAlleRegister();
      db = new Datenbasis();
      containerAuswertung = new ContainerAuswertung(this, db, seitenFormat);
      status.setText(copyright);
    }
  }
  
  private final FileFilter fileFilter = new FileFilter() {
    public boolean accept(File file) {
      if (file.isDirectory())
        return true;
      if(file.getName().toLowerCase().endsWith(".jhh"))
        return true;
      return false;
    }
    public String getDescription() {
      return "jHaushalt-Dateien (*.jhh)";
    }
  };
  
  public void laden() {
    if(abfrageGeaendert()) {
      JFileChooser dateidialog = new JFileChooser();
      dateidialog.setFileFilter(fileFilter);
      dateidialog.setCurrentDirectory(new File(properties.getProperty("jhh.ordner")));
      if(dateidialog.showOpenDialog(frame) == JFileChooser.APPROVE_OPTION)
        laden(dateidialog.getSelectedFile());
    }
  }
  
  private void laden(File datei) {
    entferneAlleRegister();
    db = new Datenbasis();
    db.setStartDatum(new Datum(properties.getProperty("jhh.opt.startdatum", "01.01.00")));
    try {
      FileInputStream fis = new FileInputStream(datei);
      ProgressMonitorInputStream pmis = new ProgressMonitorInputStream(
          frame, "Lese " + datei + " ...", fis);
      DataInputStream in = new DataInputStream(pmis);
      String versionInfo = in.readUTF();
      if((!versionInfo.equals("jHaushalt"+version)) &&
          (JOptionPane.showConfirmDialog(null,
              "Sie laden Daten die mit "+versionInfo+" gespeichert wurden.\n"+
              "Wenn Sie die Daten mit dieser Version ("+version+") speichern,\n"+
              "knnen Sie nicht mehr zur alten Version zurckkehren.\n"+
              "Fortsetzen?",
              "Achtung",
              JOptionPane.YES_NO_OPTION) == JOptionPane.NO_OPTION))
        fis.close();
      else {
        db.laden(in, versionInfo);
        fis.close();
        properties.setProperty("jhh.dateiname", datei.getPath());
        status.setText(datei.getPath()+" geladen.");
      }
    } catch(FileNotFoundException e) {
      status.setText("-E- "+datei.getPath()+" nicht gefunden!");
      properties.remove("jhh.dateiname");
    } catch(IOException e) {
      status.setText("-E- Fehler beim Laden: "+datei.getPath());
      properties.remove("jhh.dateiname");
    }
    int anzahl = db.autoBuchungenAusfuehren();
    if(anzahl > 0)
      status.setText("Es wurden "+anzahl+" automatische Buchung ausgefhrt.");
    zeigeAlleRegister();
    
    // Auswertungen laden
    if(DEBUG)
      System.out.println("Initialisiere die Auswertungen.");
    containerAuswertung = new ContainerAuswertung(this, db, seitenFormat);
    String dateiname = datei.getPath() + ".jha";
    containerAuswertung.laden(dateiname);
  }
  
  public void speichern() {
    String dateiname = properties.getProperty("jhh.dateiname");
    if(DEBUG)
      System.out.println("Speichern: Dateiname="+dateiname);
    if(dateiname == null)
      speichernUnter();
    else
      speichern(new File(dateiname));
  }
  
  private void speichern(File datei) {
    try {
      if(!datei.getName().toLowerCase().endsWith(".jhh")) {
        String name = datei.getAbsolutePath()+".jhh";
        datei = new File(name);
      }
      FileOutputStream fos = new FileOutputStream(datei);
      DataOutputStream out = new DataOutputStream(fos);
      db.speichern(out);
      out.flush();
      fos.close();
      properties.setProperty("jhh.dateiname", datei.getPath());
      status.setText(datei.getPath()+" gespeichert.");
    }
    catch(FileNotFoundException e1) {
      status.setText("-E- "+datei.getPath()+" nicht gefunden!");
    }
    catch(IOException e2) {
      status.setText("-E- Fehler beim Speichern: "+datei.getPath());
    }
    
    // Speichern der Auswertungen
    containerAuswertung.speichern(datei.getPath() + ".jha");
    
  }
  
  public void speichernUnter() {
    JFileChooser dateidialog = new JFileChooser();
    dateidialog.setFileFilter(fileFilter);
    dateidialog.setCurrentDirectory(new File(properties.getProperty("jhh.ordner")));
    if(dateidialog.showSaveDialog(frame) == JFileChooser.APPROVE_OPTION)
      speichern(dateidialog.getSelectedFile());
  }
  
  public void beenden() {
    if(abfrageGeaendert()) {
      Dimension dimension = tabbedPane.getSize();
      properties.setProperty("jhh.register.breite", ""+dimension.width);
      if(dimension.height > 100)
        properties.setProperty("jhh.register.hoehe", ""+dimension.height);
      else
        properties.setProperty("jhh.register.hoehe", "100");
      if(columnModel != null)
        for(int i=0; i<columnModel.getColumnCount(); i++)
          properties.setProperty("jhh.register.spalte"+i, ""+columnModel.getColumn(i).getWidth());
      
      // Speichert die individuellen Programmeigenschaften in die Datei "<i>.jhh</i>".
      String userHome = System.getProperty("user.home");
      File datei = new File(userHome, ".jhh");
      try {
        FileOutputStream fos = new FileOutputStream(datei);
        properties.store(fos, "Properties: "+version);
        fos.close();
      }
      catch(FileNotFoundException e1) {
        if(DEBUG)
          System.out.println("-W- 'Datei nicht gefunden' beim Spiechern der Properties.");
      }
      catch (IOException e) {
        if(DEBUG)
          System.out.println("-W- Properties konnten nicht ins Home-Verzeichnis geschrieben werden.");
      }
      
      // Ende :-(
      if(DEBUG)
        System.out.println("Ende :-(");
      System.exit(0);
    }
  }
  
  // === Men: Bearbeiten ======================================================
  
  public void suchen() {
    if(keinRegisterVorhanden()) 
      return;
    dlgSuchenErsetzen.showDialog(db);
  }
  
  public void umbuchen() {
    if(keinRegisterVorhanden()) 
      return;
    int registerIndex = tabbedPane.getSelectedIndex();
    String regname = tabbedPane.getTitleAt(registerIndex);
    GenerischerDialog dlg = new GenerischerDialog("Umbuchung eingeben", frame);
    DatumGDP pane1 = new DatumGDP("Bitte geben Sie ein Datum ein:", new Datum());
    dlg.addPane(pane1);
    TextGDP pane2 = new TextGDP("Bitte geben Sie den Buchungstext ein:", "<Umbuchung>");
    dlg.addPane(pane2);
    RegisterGDP pane3 = new RegisterGDP("Bitte whlen Sie ein Quell-Register aus:", db.getRegisterNamen(), regname);
    dlg.addPane(pane3);
    RegisterGDP pane4 = new RegisterGDP("Bitte whlen Sie ein Ziel-Register aus:", db.getRegisterNamen(), regname);
    dlg.addPane(pane4);
    EuroGDP pane5 = new EuroGDP("Bitte geben Sie ein Betrag ein:", new Euro());
    dlg.addPane(pane5);
    if(dlg.showDialog()) {
      Datum datum = (Datum)pane1.getWert();
      String text = ""+pane2.getWert();
      String quelle = ""+pane3.getWert();
      String ziel = ""+pane4.getWert();
      Euro betrag = (Euro)pane5.getWert();
      db.umbuchungEinfuegen(datum, text, quelle, ziel, betrag);
      registerVeraendert(getTabNr(quelle));
      registerVeraendert(getTabNr(ziel));
    }
  }
  
  public void loeschen() {
    if(keinRegisterVorhanden()) 
      return;
    int registerIndex = tabbedPane.getSelectedIndex();
    JScrollPane scrollPane = (JScrollPane) tabbedPane.getSelectedComponent();
    String regname = tabbedPane.getTitleAt(registerIndex);
    JTable table = (JTable) scrollPane.getViewport().getView();
    int buchungIndex = table.getSelectedRow();
    if(buchungIndex == -1) {
      JOptionPane.showMessageDialog(frame,
      "Es ist keine Zeile zum Lschen selektiert.");
      return;
    }
    if(buchungIndex == db.getAnzahlBuchungen(regname)) {
      JOptionPane.showMessageDialog(frame,
      "Die letzte (Eingabe-)Zeile kann nicht gelscht werden.");
      return;
    }
    db.entferneBuchung(regname, buchungIndex);
    RegisterTableModel tableModel = (RegisterTableModel) table.getModel();
    tableModel.fireTableRowsDeleted(buchungIndex, buchungIndex);
    if(table.getCellEditor() != null)
      table.getCellEditor().cancelCellEditing();
    table.requestFocus();
    table.setRowSelectionInterval(buchungIndex, buchungIndex);
    status.setText("Buchung Nr. "+(buchungIndex+1)+" im Register "+regname+" gelscht.");
  }
  
  public void splitten() {
    if(keinRegisterVorhanden()) 
      return;
    int registerIndex = tabbedPane.getSelectedIndex();
    JScrollPane scrollPane = (JScrollPane) tabbedPane.getSelectedComponent();
    String regname = tabbedPane.getTitleAt(registerIndex);
    JTable table = (JTable) scrollPane.getViewport().getView();
    int buchungIndex = table.getSelectedRow();
    if(buchungIndex == -1) {
      JOptionPane.showMessageDialog(frame,
      "Es ist keine Buchung zum Splitten selektiert!");
      return;
    }
    AbstractBuchung buchung = db.getBuchung(regname, buchungIndex);
    if(buchung.getClass() == Umbuchung.class) {
      JOptionPane.showMessageDialog(frame,
      "Umbuchungen knnen nicht gesplittet werden.");
    }
    else {
      SplitBuchung splitBuchung;
      if(buchung.getClass() == StandardBuchung.class)
        splitBuchung = new SplitBuchung((StandardBuchung)buchung);
      else
        splitBuchung = (SplitBuchung) buchung;
      JDialog dlg = new DlgSplitBuchung(this, db, splitBuchung);
      dlg.pack();
      dlg.setVisible(true);
      int pos = db.ersetzeBuchung(regname, buchungIndex, splitBuchung.reduziere());
      db.buchungMerken(splitBuchung);
      registerVeraendert(getTabNr(regname));
      selektiereBuchung(regname, pos);
    }
    if(table.getCellEditor() != null)
      table.getCellEditor().cancelCellEditing();
    table.requestFocus();
    if(DEBUG)
      System.out.println("Buchung Nr. "+buchungIndex+" im Register "+regname+" gesplittet.");
  }
  
  public void neuesRegister() {
    GenerischerDialog dlg = new GenerischerDialog("Neues Register", frame);
    TextGDP pane = new TextGDP("Name des neuen Registers:", "NEU");
    dlg.addPane(pane);
    
    if(dlg.showDialog()) {
      String name = db.erzeugeRegister("["+pane.getWert()+"]");
      // falls der Name doppelt ist, hat er eine folgende Ziffer erhalten
      zeigeRegister(name);
      status.setText("Register "+name+" erzeugt.");
    }
  }
  
  public void kategorienBearbeiten() {
    DlgKategorienBearbeiten dlg = new DlgKategorienBearbeiten(this, db);
    dlg.showDialog();
    alleRegisterVeraendert();
  }
  
  public void alteBuchungenLoeschen() {
    GenerischerDialog dlg = new GenerischerDialog("Alte Buchungen lschen", frame);
    DatumGDP pane = new DatumGDP("Datum bis zu dem alle Buchungen gelscht werden:", new Datum());
    pane.setPreferredSize(new Dimension(320,60));
    dlg.addPane(pane);
    
    if(dlg.showDialog()) {
      Datum datum = (Datum)pane.getWert();
      db.entferneAlteBuchungen(datum);
      alleRegisterVeraendert();
      status.setText("Alle Buchungen bis zum "+datum+" gelscht.");
    }
  }
  
  public void kategorieErsetzen() {
    GenerischerDialog dlg = new GenerischerDialog("Kategorie ersetzen", frame);
    EineKategorieGDP pane1 = new EineKategorieGDP("Alte Kategorie:", db.getKategorien(), null);
    dlg.addPane(pane1);
    EineKategorieGDP pane2 = new EineKategorieGDP("Neue Kategorie:", db.getKategorien(), null);
    dlg.addPane(pane2);
    
    if(dlg.showDialog()) {
      Kategorie alteKategorie = (Kategorie)pane1.getWert();
      Kategorie neueKategorie = (Kategorie)pane2.getWert();
      int anzahl = db.ersetzeKategorie(alteKategorie, neueKategorie);
      status.setText(""+anzahl+" Kategorien ersetzt.");
      alleRegisterVeraendert();
    }
  }
  
  public void registerVereinigen() {
    if(keinRegisterVorhanden()) 
      return;
    int registerIndex = tabbedPane.getSelectedIndex();
    String regname = tabbedPane.getTitleAt(registerIndex);
    GenerischerDialog dlg = new GenerischerDialog("Register vereinigen ...", frame);
    RegisterGDP pane1 = new RegisterGDP(
        "Bitte whlen Sie ein Register zum Lschen aus (Quelle):",
        db.getRegisterNamen(), regname);
    pane1.setPreferredSize(new Dimension(330,60));
    dlg.addPane(pane1);
    RegisterGDP pane2 = new RegisterGDP(
        "Bitte whlen Sie ein Register zum Einfgen aus (Ziel):",
        db.getRegisterNamen(), regname);
    pane2.setPreferredSize(new Dimension(330,60));
    dlg.addPane(pane2);
    
    if(dlg.showDialog()) {
      String quelle = (String)pane1.getWert();
      String ziel = (String)pane2.getWert();
      if(!quelle.equals(ziel)) {
        db.registerVereinigen(quelle, ziel);
        entferneRegister(quelle);
        status.setText("Register "+quelle+" gelscht.");
        registerVeraendert(getTabNr(ziel));
      }
      else
        JOptionPane.showMessageDialog(frame, "Quell- und Zielregister drfen nicht gleich sein.");
    }
  }
  
  // == Men: Ausgabe ==========================================================
  
  public void zeigeAuswertung() {
    if(keinRegisterVorhanden()) 
      return;
    containerAuswertung.zeigeDialog();
  }
  
  public void druckerEinrichten() {
    seitenFormat = PrinterJob.getPrinterJob().pageDialog(seitenFormat);
  }
  
  // == Men: Extras ===========================================================
  
  public void optionen() {
    dlgOptionen.showDialog();
    oberflaecheAnpassen();
  }
  
  public void autoBuchung() {
    if(keinRegisterVorhanden()) 
      return;
    DlgAutoBuchung dlg = new DlgAutoBuchung(this, db);
    dlg.zeigeDialog();
    alleRegisterVeraendert();
  }
  
  public void importCSV() {
    if(keinRegisterVorhanden()) 
      return;
    int registerIndex = tabbedPane.getSelectedIndex();
    String regname = tabbedPane.getTitleAt(registerIndex);
    CsvHandler handler = new CsvHandler();
    if(handler.importDlg(frame, getOrdner())) {
	    String[][] tabelle = handler.getTabelle();
	    if(tabelle[0].length == 4) {
	      db.importBuchungen(regname, tabelle);
	      status.setText(""+tabelle.length+" Buchungen importiert.");
	      registerVeraendert(registerIndex);
	    }
	    else {
	      JOptionPane.showMessageDialog(frame,
	          "Die Datei enthlt "+tabelle[0].length+" statt 4 Spalten.");
	    }
    }
  }
  
  public void importQuicken() {
    if(keinRegisterVorhanden()) 
      return;
    int registerIndex = tabbedPane.getSelectedIndex();
    String regname = tabbedPane.getTitleAt(registerIndex);
    GenerischerDialog dlg = new GenerischerDialog("QIF Import", frame);
    String[] registerNamen = db.getRegisterNamen();
    RegisterGDP pane = new RegisterGDP(
        "Bitte whlen Sie ein Register aus:", registerNamen, regname);
    pane.setPreferredSize(new Dimension(250,60));
    dlg.addPane(pane);
    if(dlg.showDialog()) {
      JFileChooser dateidialog = new JFileChooser();
      dateidialog.setCurrentDirectory(new File(properties.getProperty("jhh.ordner")));
      if(dateidialog.showOpenDialog(frame) == JFileChooser.APPROVE_OPTION) {
        File datei = dateidialog.getSelectedFile();
        try {
          FileInputStream in = new FileInputStream(datei);
          boolean euroImport = Boolean.valueOf(properties.getProperty("jhh.opt.euroimport", "true")).booleanValue();
          db.importQuickenRegister(
              in, 
              (String)pane.getWert(),
              euroImport 
          );
          in.close();
          zeigeAlleRegister();
        } catch (FileNotFoundException e1) {
          if(DEBUG)
            System.out.println("-E- "+datei.getPath()+" nicht gefunden!");
        } catch (IOException e2) {
          if(DEBUG)
            System.out.println("-E- Fehler beim Importieren: "+datei.getPath());
        } catch (QuickenImportException e) {
          if(DEBUG)
            System.out.println("-E- Fehler beim Importieren: "+datei.getPath());
          status.setText(e.getMessage());
        }
      }
    }
  }
  
  // == Men: Hilfe ============================================================
  
  /**
   * Wird von Men-Handler bei Auswahl von '<b>Hilfe/Inhalt</b>' aufgerufen.
   * Es wird standardmig die Datei "html/help.html" angezeigt.
   */
  public void hilfeInhalt() {
    DlgHilfe dlg = new DlgHilfe(frame);
    dlg.pack();
    dlg.setVisible(true);
  }
  
  /**
   * Wird von Men-Handler bei Auswahl von '<b>Hilfe/Programm-Info</b>' aufgerufen.
   */
  public void programmInfo() {
    DlgInfo dlg = new DlgInfo(frame, copyright);
    dlg.pack();
    dlg.setVisible(true);
  }
  
  // -- Methoden des Interface 'KeyListener' -----------------------------------
  
  public void keyReleased(KeyEvent e) {
    if(e.getSource().getClass() == DeleteableTextField.class) {
      DeleteableTextField textField =(DeleteableTextField)e.getSource();
      setzteKoordinatenGemerkteBuchung(textField);
      zeigeGemerkteBuchung(textField.getText());        
    }
  }
  
  public void keyPressed(KeyEvent e){
    // nichts zu tun !
  }
  
  public void keyTyped(KeyEvent e){
    // nichts zu tun !
  }
  
  // -- Methoden des Interface 'ListSelectionListener' ----------------------------
  
  public void valueChanged(ListSelectionEvent e) {
    glassPane.setVisible(false);
  }
  
  // -- MAIN -------------------------------------------------------------------
  
  public static void main(String[] args) {
    String version = System.getProperty("java.specification.version");
    if(version.compareTo("1.4") < 0) {
      JOptionPane.showMessageDialog(null,
        "Sie verwenden Java "+version+".\n"+
        "Bitte installieren Sie Java >= 1.4,\n"+
        "um jHaushalt auszufhren.");
      System.exit(0);
    }
    String dateiname = null;
    if((args.length > 0) && (!args[0].equals("")))
      dateiname = args[0];
    Haushalt haushalt = new Haushalt(dateiname);
    JFrame frame = haushalt.getFrame();
    frame.pack();
    frame.setVisible(true);
  }
  
}