/*

This file is part of jHaushalt.

jHaushalt is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

jHaushalt is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with jHaushalt; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

(C)opyright 2002-2005 Dr. Lars H. Hahn

*/

package haushalt.gui;

import haushalt.daten.Datenbasis;
import haushalt.daten.Kategorie;

import java.awt.Component;
import java.util.ArrayList;

import javax.swing.DefaultCellEditor;
import javax.swing.JComboBox;
import javax.swing.JTable;

/**
 * Cell-Editor fr die Kategorie.
 * @author Dr. Lars H. Hahn
 * @version 2.0/2004.08.22
 */


public class KategorieCellEditor extends DefaultCellEditor {
  private final boolean DEBUG = false;

  private final Haushalt haushalt;
  private final Datenbasis db;
  private JComboBox comboBox;
  private Object[] registerUmbuchung = null;
  private int registerNr = 0;

  public KategorieCellEditor(Haushalt haushalt, Datenbasis db) {
    super(new JComboBox());
    comboBox = (JComboBox) getComponent();
    this.haushalt = haushalt;
    this.db = db;
    setClickCountToStart(1);
  }

  public Object getCellEditorValue() {
    Object object = comboBox.getSelectedItem();
    if(DEBUG) 
      System.out.println("KategorieCellEditor: getCellEditorValue = "+object);
    if(object.getClass() == Kategorie.class)
      return object;
    // else: Umbuchung (SplitBuchungen werden vorher umgeleitet.)
    Object neuesRegister = db.findeOderErzeugeRegister(""+object);
    registerUmbuchung[registerNr] = neuesRegister;
    haushalt.alleRegisterVeraendert();
    return registerUmbuchung;
  }

  public Component getTableCellEditorComponent(JTable table, Object value, boolean isSelected, int row, int col) {
		comboBox = (JComboBox) super.getTableCellEditorComponent(table, value, isSelected, row, col);
		if(value == null)
		  value = Kategorie.SONSTIGES;
		if(value.getClass() == Kategorie.class) {
			comboBox = new JComboBox(db.getKategorien());
			comboBox.setSelectedItem(value);
		}
		else if(value.getClass() == ArrayList.class) {
			ArrayList list = (ArrayList)value;
			Object object = list.get(0);
			if(object.getClass() == Kategorie.class) {
				haushalt.selektiereBuchung(""+table.getModel(), row);
	 			haushalt.splitten();
	 			comboBox = null;
				cancelCellEditing();
			}
    }
		else if(value.getClass().isArray()) {
		  registerUmbuchung = (Object[])((Object[]) value).clone();
			String regname = ""+table.getModel();
			comboBox = new JComboBox(db.getRegisterNamen());
			// Ob das Quell- oder Ziel-Register in der Combo-Box
			// ausgewhlt werden muss, ist davon abhngig in
			// welchem der beiden Register sich die Umbuchung
			// befindet. 
			if(regname.equals(""+registerUmbuchung[0])) // = Quell-Register
			  registerNr = 1; // also Ziel-Register auswhlen
			else
			  registerNr = 0; // also Quell-Register auswhlen
		  comboBox.setSelectedItem(""+registerUmbuchung[registerNr]);				
		}
		else if(DEBUG)
			System.out.println("-E- KategorieCellEditor: Unerwartete Klasse: "+value.getClass());
		return comboBox;
  }
  
}