/*

This file is part of jHaushalt.

jHaushalt is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

jHaushalt is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with jHaushalt; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

(C)opyright 2002-2005 Dr. Lars H. Hahn

*/

/*
 * 2005.02.18 BugFix: Das bernehmen einer auto. Buchung kann zum Neusortieren
 * fhren, dies wurde aber nicht angezeigt und es wurde nicht mitgesprungen.
 * 2004.08.25 BugFix: Beim Einfgen einer neuen Buchung wurde teilweise, die 
 * zuvor erzeugte berschrieben.
 * 2004.08.25 BugFix: Bei der Verwendung von gemerkten Buchungen wurde teilweise 
 * die Anzeige nicht aktualisiert.
 */

package haushalt.gui;

import haushalt.daten.AbstractBuchung;
import haushalt.daten.Datenbasis;
import haushalt.daten.Datum;
import haushalt.daten.Euro;
import haushalt.daten.Kategorie;
import haushalt.daten.StandardBuchung;
import haushalt.daten.Umbuchung;

import javax.swing.table.AbstractTableModel;

/**
 * Ermglicht die Darstellung eines Registers in einer Swing-Tabelle.
 * @author Dr. Lars H. Hahn
 * @version 2.02/2005.02.18
 */

public class RegisterTableModel
  extends AbstractTableModel {
  private final boolean DEBUG = false;

  private final String[] spaltenNamen =
    {"Datum","Buchungstext","Kategorie","Betrag","Saldo"};

  private final Haushalt haushalt;
  private final Datenbasis db;
  private final String regname;
  
  public RegisterTableModel(Haushalt haushalt, Datenbasis db, String name) {
    this.haushalt = haushalt;
  	this.db = db;
    this.regname = name;
  }

	public String toString() {
		return regname;
	}

	public void entferneBuchung(int row) {
		db.entferneBuchung(regname, row);
		this.fireTableRowsDeleted(row, row);
	}
	
  public int getColumnCount() {
    return spaltenNamen.length;
  }

  public int getRowCount() {
  	// eine Zeile mehr, da in der letzten Zeile die Eingabe
  	// mglich ist
    return db.getAnzahlBuchungen(regname) + 1;
  }

  public String getColumnName(int col) {
    return spaltenNamen[col];
  }

  public Class getColumnClass(int col) {
  	switch(col) {
  		case 0 : return Datum.class;
  		case 1 : return String.class; 
  		case 2 : return Object.class;
  		default: return Euro.class;
  	}
  }

  public Object getValueAt(int row, int col) {
    if(DEBUG)
      System.out.println("RegisterTableModel: getValue @ "+row+", "+col);
  	if(row < db.getAnzahlBuchungen(regname)) {
  	  AbstractBuchung buchung = db.getBuchung(regname, row);
  		switch(col) {
  			case 0 : return buchung.getDatum();
  			case 1 : return buchung.getText();
  			case 2 : return buchung.getKategorie();
  			case 3 : 
  			  if(buchung.getClass() == Umbuchung.class) {
  			    Object[] register = (Object[]) buchung.getKategorie();
  			    if(regname.equals(""+register[0]) && (register[0] != register[1]))
  			      return Euro.NULL_EURO.sub(buchung.getWert());
  			  }
 			    return buchung.getWert();
  			default: return db.getRegisterSaldo(regname, row); 
  		}
  	}
  	
  	// Werte fr die letzte Zeile gibt es noch nicht:
  	switch(col) {
  		case 0: return new Datum();
  		case 1: return new String();
  		case 2: return Kategorie.SONSTIGES;
  		case 3: return new Euro();
  		default: return db.getRegisterSaldo(regname, row-1);
  	}
  }

  public boolean isCellEditable(int row, int col) {
    if(col == 4)
      return false;
    return true;
  }

  public void setValueAt(Object value, int row, int col) {
    if(DEBUG)
      System.out.println("RegisterTableModel: setValue ("+value+") @ "+row+", "+col);
    AbstractBuchung buchung;
		if(row == db.getAnzahlBuchungen(regname)) {
      // Wenn ein Wert in der letzten Zeile eingegeben wurde,
      // muss eine neue Buchung erzeugt werden.
		  buchung = new StandardBuchung();
      row = db.standardBuchungEinfuegen(regname, (StandardBuchung)buchung);
      fireTableRowsInserted(row,row);
    }
		else
		  buchung = db.getBuchung(regname, row);
		
		int pos;
		switch(col) {
			case 0: 
        buchung.setDatum(new Datum(""+value));
				pos = db.buchungNeusortieren(regname, row);
				fireTableDataChanged();
				haushalt.selektiereBuchung(regname, pos);
				break;
			case 1:
				AbstractBuchung gemerkteBuchung = db.findeGemerkteBuchung(""+value);
				if(haushalt.gemerkteBuchungen() && (gemerkteBuchung != null)) {
				  AbstractBuchung neueBuchung = (AbstractBuchung) gemerkteBuchung.clone();
				  neueBuchung.setDatum(buchung.getDatum());
				  if(!buchung.getWert().equals(Euro.NULL_EURO))
				    neueBuchung.setWert(buchung.getWert());
				  pos = db.ersetzeBuchung(regname, row, neueBuchung);
          fireTableDataChanged();
          haushalt.selektiereBuchung(regname, pos);
				}
				else {
					buchung.setText(""+value);
					db.buchungMerken(buchung);
				}
				break;
			case 2:
        buchung.setKategorie(value);
				db.buchungMerken(buchung);
				break;
			case 3: 
        buchung.setWert(new Euro(""+value));
				db.buchungMerken(buchung);
				pos = db.buchungNeusortieren(regname, row);
				fireTableDataChanged();
				haushalt.selektiereBuchung(regname, pos);
				break;
		}  	
  }

}