/*

This file is part of jHaushalt.

jHaushalt is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

jHaushalt is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with jHaushalt; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

(C)opyright 2002-2005 Dr. Lars H. Hahn

*/

package haushalt.gui.generischerdialog;

import haushalt.gui.DeleteableTextField;

import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.IOException;

import javax.swing.text.AttributeSet;
import javax.swing.text.BadLocationException;
import javax.swing.text.Document;
import javax.swing.text.PlainDocument;
import javax.swing.JComponent;

/**
 * @author Dr. Lars H. Hahn
 * @version 2.0/2004.08.22
 */

public class ZahlGDP extends AbstractGDPane {
	private final boolean DEBUG = false;

  private final DeleteableTextField textField;

  public ZahlGDP(String textAufforderung, Integer wertEingabe) {
    super(textAufforderung);
    textField = new DeleteableTextField(""+wertEingabe, 4) {
      protected Document createDefaultModel() {
        return new ZahlDocument();
      }
    };
    add(textField);
    refreshWert();
  }

  protected void refreshWert() {
    wert = new Integer(textField.getText());
  }

  public JComponent getZentraleKomponente() {
    return textField;
  }

  public void laden(DataInputStream in) throws IOException {
    textField.setText(in.readUTF());
    refreshWert();
  }

  public void speichern(DataOutputStream out) throws IOException {
    refreshWert();
    out.writeUTF(textField.getText());
  }

  private class ZahlDocument extends PlainDocument {
    public void insertString(int offs, String str, AttributeSet a)
      throws BadLocationException {
      char[] source = str.toCharArray();
      char[] result = new char[source.length];
      int j = 0;

      for (int i = 0; i < source.length; i++) {
        if (Character.isDigit(source[i]))
          result[j++] = source[i];
        else if(DEBUG)
					System.out.println("-I- Falsches Zeichen: " + source[i]);
      }

      super.insertString(offs, new String(result, 0, j), a);
    }
  }

}